%macro build_episode_data_pre(cyear);

%let year = &cyear.;

*****************************************;
*            LOAD BSF DATA              *;
*****************************************;
*                                       *;					
* This program loads the BSF file and   *;
* renames the key variables.            *;
*                                       *;
*****************************************;

data temp.bsf_loaded&year.
	(keep = bene_: buyin: hmo: msc:);
	set mc.denom&year.
	(rename = (sdod = bene_death_dt
		sdob = bene_dob
		sex = bene_sex
		race = bene_race
		age = bene_age
		county = bene_cnty_cd
		state = bene_state_cd
		oentitl = bene_orec
		centitl = bene_crec
		stbuyin = buyin_mo
		zipcode = bene_zip
		hmocov = hmo_mo
		entitl1 = buyin1
		entitl2 = buyin2
		entitl3 = buyin3
		entitl4 = buyin4
		entitl5 = buyin5
		entitl6 = buyin6
		entitl7 = buyin7
		entitl8 = buyin8
		entitl9 = buyin9
		entitl10 = buyin10
		entitl11 = buyin11
		entitl12 = buyin12));
run;

***********************************************************;
*                 LOAD ALL LEJR ADMISSIONS                *;
***********************************************************;
*                                                         *;
* This program loads the IP file, merges in flags for hip *;
* fracture, collapses the data to the stay level, and     *;
* creates the sample of LEJR admissions.                  *; 
*                                                         *;
***********************************************************;

** LOAD IP DATA **;
* Note: Medicare and out-of-pocket payments are defined based on                     *;
* http://resdac.umn.edu/sites/resdac.umn.edu/files/                                  *;
* Definitions%20of%20%27Cost%27%20in%20Medicare%20Utilization%20Files%20(Slides).pdf *;

data tempfile
	(keep = admsn_dt dschrgdt pmt_amt util_day per_diem
	ded_amt coin_amt icd_cd blddedam pdgns_cd bene_id
	drg_cd pps_ind provider stus_cd prpay_cd oop_amt);
	set mc.ip&year.clm 
	(rename = (sadmsndt = admsn_dt sdschrgdt = dschrgdt));
	pmt_amt = pmt_amt + util_day*per_diem ;
	if pmt_amt < 0 then pmt_amt = 0 ;
	oop_amt = ded_amt + coin_amt + blddedam ;

	if &year.<=2015 then do;
		* remove leading 0s *;
		icd_cd_num = compress(pdgns_cd,'','a');
		icd_cd_int = input(icd_cd_num, 7.);
		icd_cd = put(icd_cd_int, $7.);
	end;
	else do;
		icd_cd = pdgns_cd;
	end;
run;

proc sort data = tempfile out=tempfile;
	by icd_cd;
quit;

** CREATE FLAGS FOR HIP FRACTURE **;
* Hip fracture icd codes downloaded from CJR website *;
* https://innovation.cms.gov/Files/worksheets/cjr-icd10hipfracturecodes.xlsx *;

data temp.hip_fracture_icd_loaded (keep = icd_cd hip_fracture_clm);
	if &year. <= 2015 then do;
		set temp.hip_fracture_icd9;
		icd_cd = put(icd9_cd, 7.);
		hip_fracture_clm = 1;
	end;
	* Our data ends by Sept 15 so in IP can ignore the ICD-9 to 10 switch for 2015;
	if &year. >= 2016 then do;
		set temp.hip_fracture_icd10;
		icd_cd = compress(icd10_cd,".");
		hip_fracture_clm = 1;
	end;
run;

proc sort data = temp.hip_fracture_icd_loaded out=temp.hip_fracture_icd_loaded;
	by icd_cd;
quit;

** COLLAPSE IP DATA TO STAY LEVEL **;
** per https://www.resdac.org/resconnect/articles/125 **;

proc sql nowarn;
	create table tempfile2 as
	select A.*, B.*
	from tempfile as A
	left join
	temp.hip_fracture_icd_loaded as B
	on A.icd_cd=B.icd_cd;
quit;
		
proc sort data = tempfile2 out = tempfile3;
	by bene_id admsn_dt provider dschrgdt;
quit;

data temp.ip_stay_level_loaded&year. 
	(keep = ipf ach ltch irf bene_id admsn_dt dschrgdt provider prvdrnum payment oop
        stus_cd mcareprimary pps_ind drg_cd hip_fracture_flag); 
	
	set tempfile3;
	by bene_id admsn_dt provider dschrgdt;
	retain payment oop mcareprimary hip_fracture_flag;
	
	mcareprimary_clm = (prpay_cd in ("", "Z"));
	
	if first.provider then do;
		payment = pmt_amt;
		oop = oop_amt; 
		mcareprimary = mcareprimary_clm ;
		hip_fracture_flag = hip_fracture_clm;
	end;
	
	else do;
		payment = payment + pmt_amt;
		oop = oop + oop_amt;
		if mcareprimary_clm < mcareprimary then mcareprimary = mcareprimary_clm;
		if hip_fracture_flag < hip_fracture_clm then hip_fracture_flag = 1;
	end;
	
	
	if last.provider;
	
	prvdstr = substr(provider, 3, 4);

        * FLAGS FOR DIFFERENT FACILITY TYPES *;
	* per https://www.resdac.org/sites/resdac.umn.edu/files/Provider%20Number%20Table.txt *;
	* only keep those with only numeric values in the last four digits; 

	if anyalpha(prvdstr) = 0 ; 
	prvdrnum = input(prvdstr, 4.);

	ach  = ((prvdrnum <= 999 & ~missing(prvdrnum)) | (prvdrnum >= 1300 & prvdrnum <= 1399)) ;
	ltch = (prvdrnum <= 2299 & prvdrnum >= 2000) ;
	irf  = (prvdrnum <= 3099 & prvdrnum >= 3025) ;
	ipf  = (prvdrnum <= 4499 & prvdrnum >= 4000) ;
	
	* EXCLUDE IF STILL A PATIENT *;

	if ~missing(dschrgdt) and stus_cd ~= "30";
	if missing(hip_fracture_flag) then hip_fracture_flag = 0;
run;

** RESTRICT TO LEJR SAMPLE AND CREATE LIST OF BENES **;
* ACH admissions for DRG 469&470 bewteen APR 1 and SEPT 15 *;
* Episode begins with the LEJR admission at ACH and ends 90 days post discharge *;
* (day of discharge counts as one day) *;

data tempfile;
	set temp.ip_stay_level_loaded&year. (where = (ach = 1 and (drg_cd = "469" or drg_cd = "470"))
	rename = (payment = index_pay oop = index_oop));
	format apr1&year. date9.;
	format sept15&year. date9.;
	apr1&year. = input("01APR&year.",date9.);
	sept15&year = input("15SEP&year.",date9.);
	if admsn_dt <= sept15&year. and admsn_dt >= apr1&year.;
	los = max(dschrgdt - admsn_dt, 1);
	epsdenddt = dschrgdt + 89;
run;


proc sql nowarn;
	 create table temp.all_lejr_index&year. as 
	 select A.*, B.*
	 from tempfile as A
	 inner join 
	 temp.bsf_loaded&year. as B
	 on A.bene_id = B.bene_id;
quit;

**********************************************************;
*          DEFINE CJR LEJR INDEX ADMISSIONS              *;
**********************************************************;
*                                                        *;
* 1. Create flags for CJR sample exclusions - "drop_XX"  *;
* The exclusions are:                                    *;
*  - Not at a PPS hospital                               *;
*  - In BPCI                                             *;
*  - Not on both Parts A&B throughout episode            *;
*  - On HMO during any part of episode                   *;
*  - ESRD                                                *;
*  - Medicare not the primary payer                      *;
*  - Readmitted for LEJR (DRGs 469 or 470)               *;
*  - Death during episode                                *;
* (The entire episode is ineligible if any of the above  *;
* happens)                                               *;
* 2. Restrict to episodes that are in an MSA             *;
* and merge in flags for MSA treatment status            *;
* 3. Create dataset with all index admissions -          *;
* "cjr_lejr_index_admissions&year."                      *;
*                                                        *;
**********************************************************;

%macro merge_in_bpci_indicators();

%if &year. >= 2014 %then %do;
	proc sql nowarn;
		create table tempinput_merge as
		select A.*, B.*
		from temp.all_lejr_index&year. as A
		left join
		temp.bpci_hosps_&year. as B
		on A.provider = B.provider;
	quit;
	data tempinput;
		set tempinput_merge;

		** FLAG FOR NON-PPS HOSPITALS **;
		drop_pps = (pps_ind ~="2");

		** FLAG FOR BPCI PARTICIPATION **;
		june30&year. = input("30JUN&year.",date9.);
		july1&year. = input("01JUL&year.",date9.);
		sept30&year. = input("30SEP&year.",date9.);
		oct1&year. = input("01OCT&year.",date9.);
		dec31&year. = input("31DEC&year.",date9.);
		
        	episode_quarter2 = (admsn_dt <= june30&year. & epsdenddt >= apr1&year.);
        	episode_quarter3 = (admsn_dt <= sept30&year. & epsdenddt >= july1&year.);
        	episode_quarter4 = (admsn_dt <= dec31&year. & epsdenddt >= oct1&year.);
			
        	drop_bpci = 0;

		if episode_quarter2 = 1 & bpci_quarter2 = 1 then drop_bpci = 1;
		if episode_quarter3 = 1 & bpci_quarter3 = 1 then drop_bpci = 1;
		if episode_quarter4 = 1 & bpci_quarter4 = 1 then drop_bpci = 1;
	run;
	%end;

	%else %do;
	data tempinput;
		set  temp.all_lejr_index&year. ;
		drop_pps = (pps_ind ~= "2");
		drop_bpci = 0;
	run;
%end;
%mend;

%merge_in_bpci_indicators();

data tempfile;
	set tempinput;

	episode_mo04 = (month(admsn_dt) <= 04 & month(epsdenddt) >= 04);
	episode_mo05 = (month(admsn_dt) <= 05 & month(epsdenddt) >= 05);
	episode_mo06 = (month(admsn_dt) <= 06 & month(epsdenddt) >= 06);
	episode_mo07 = (month(admsn_dt) <= 07 & month(epsdenddt) >= 07);
	episode_mo08 = (month(admsn_dt) <= 08 & month(epsdenddt) >= 08);
	episode_mo09 = (month(admsn_dt) <= 09 & month(epsdenddt) >= 09);
	episode_mo10 = (month(admsn_dt) <= 10 & month(epsdenddt) >= 10);
	episode_mo11 = (month(admsn_dt) <= 11 & month(epsdenddt) >= 11);
	episode_mo12 = (month(admsn_dt) <= 12 & month(epsdenddt) >= 12);
	
	** FLAG FOR MEDICARE PARTS A & B **;
	drop_partAB = 0;
	if episode_mo04 = 1 and buyin4 ~= "3" and buyin4 ~= "C" then drop_partAB = 1;
	if episode_mo05 = 1 and buyin5 ~= "3" and buyin5 ~= "C" then drop_partAB = 1;
	if episode_mo06 = 1 and buyin6 ~= "3" and buyin6 ~= "C" then drop_partAB = 1;
	if episode_mo07 = 1 and buyin7 ~= "3" and buyin7 ~= "C" then drop_partAB = 1;
	if episode_mo08 = 1 and buyin8 ~= "3" and buyin8 ~= "C" then drop_partAB = 1;
	if episode_mo09 = 1 and buyin9 ~= "3" and buyin9 ~= "C" then drop_partAB = 1;
	if episode_mo10 = 1 and buyin10 ~= "3" and buyin10 ~= "C" then drop_partAB = 1;
	if episode_mo11 = 1 and buyin11 ~= "3" and buyin11 ~= "C" then drop_partAB = 1;
	if episode_mo12 = 1 and buyin12 ~= "3" and buyin12 ~= "C" then drop_partAB = 1;
	
	** FLAG FOR HMO **;
	drop_hmo = 0 ;
	if episode_mo04 = 1 & hmo4 ~= "0" then drop_hmo = 1;
	if episode_mo05 = 1 & hmo5 ~= "0" then drop_hmo = 1;
	if episode_mo06 = 1 & hmo6 ~= "0" then drop_hmo = 1;
	if episode_mo07 = 1 & hmo7 ~= "0" then drop_hmo = 1;
	if episode_mo08 = 1 & hmo8 ~= "0" then drop_hmo = 1;
	if episode_mo09 = 1 & hmo9 ~= "0" then drop_hmo = 1;
	if episode_mo10 = 1 & hmo10 ~= "0" then drop_hmo = 1;
	if episode_mo11 = 1 & hmo11 ~= "0" then drop_hmo = 1;
	if episode_mo12 = 1 & hmo12 ~= "0" then drop_hmo = 1;
	
	** FLAG FOR ESRD **;
	drop_esrd = bene_crec = "2";

	** FLAG FOR MEDICARE AS PRIMARY PAYER **;
	drop_mcareprimary = mcareprimary = 0;

	** PLACEHOLDER FOR FLAG FOR LEJR READMISSION **;
	drop_readmit = 0;

	** FLAG FOR DEATH IN EPISODE **;
	drop_die = (stus_cd = "20" or (bene_death_dt < epsdenddt AND bene_death_dt ~=.));
run;

** FLAG FOR LEJR READMISSION **;
* The flag is on the episode that has the readmission *;
* The readmission itself initiates another episode *;

proc sort data = tempfile out = tempfile2;
	by bene_id descending admsn_dt;
quit;

data tempfile3 (drop = next_admsn_dt);
	set tempfile2;
	by  bene_id descending admsn_dt;
	retain next_admsn_dt;
	
	if first.bene_id then next_admsn_dt = .;
	drop_readmit = 0;
		
	if ~missing(next_admsn_dt) and next_admsn_dt <= epsdenddt then drop_readmit = 1;
		
	next_admsn_dt = admsn_dt;
run;

** MERGE IN MSAS AND FLAGS FOR ELIGIBLE, CONTROL, AND TREATMENT MSAS **;
* Restrict to episodes within MSAs *;
* Use the same provider-to-MSA crosswalk for all years *;

proc sql nowarn;
	create table tempfile4 as
	select A.*, B.*
	from tempfile3 as A
	inner join temp.provider_to_msa_xwalk2016 as B
	on A.provider = B.provider;
quit;

proc sql nowarn;
	create table tempfile5 as
	select A.*, B.*
	from tempfile4 as A
	left join temp.treatment_eligible_strata as B
	on A.msa = B.msa;
quit;

proc sort nodupkey data = tempfile5;
	by bene_id admsn_dt;
quit;

** CREATE DATASET OF INDEX ADMISSIONS WITH UNIQUE EPISODE IDENTIFIERS **;

data temp.cjr_lejr_index_admissions&year.;
	set tempfile5;
	by bene_id admsn_dt;
	match_id = _n_;
	episode_id = cat(bene_id, "_",match_id);
run;

proc sort nodupkey data = temp.cjr_lejr_index_admissions&year. (keep = bene_id)
	out = temp.cjr_lejr_bene_list&year.;
	by bene_id;
quit;


**************************************************************************************;
*                             OTHER UTILIZATION IN EPISODE                           *;
**************************************************************************************;
*                                                                                    *;
* Load other (non LEJR) utilization from:                                            *;
*   - ACH                                                                            *;
*   - LTCH                                                                           *;
*   - IRF                                                                            *;
*   - IPF                                                                            *;
*   - SNF                                                                            *;
*   - HHA                                                                            *;
*   - HOS                                                                            *;
*   - DME                                                                            *;
*   - OP                                                                             *;
*   - PTB/CAR                                                                        *;
*                                                                                    *;
* For each file, do the following                                                    *;
* (0) Load any variables not in the claims file (if applicable)                      *;
* (1) Restrict to benes who had an index admission - "cjr_lejr_bene_list&year."      *;
* (2) Apply service exclusions (if applicable)                                       *;
* (3) Rename variables for admission date, discharge date, payment, out-of-pocket    *;
* payment, length of stay.                                                           *;
* (4) Save file as &type._loaded&year.                                               *;
*                                                                                    *;
* Note: Medicare and out-of-pocket payments are defined based on                     *;
* http://resdac.umn.edu/sites/resdac.umn.edu/files/                                  *;
* Definitions%20of%20%27Cost%27%20in%20Medicare%20Utilization%20Files%20(Slides).pdf *;
*                                                                                    *;
**************************************************************************************;

** ACH **;

proc sql nowarn;
	create table tempfile as
	select A.*, B.*
	from temp.ip_stay_level_loaded&year. (where = (ach=1 and drg_cd~="469" and drg_cd~="470")) as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id=B.bene_id;
quit;
 
data temp.ip_readmission_exclusions_loaded;
	set temp.ip_readmission_exclusions;
	excl_ach = 1;	
	drg_num = input(drg_cd, 3.);
run;

data tempfile2;
    set tempfile;
	drg_num = input(drg_cd, 3.);
run;

proc sql nowarn;
	create table tempfile3 as
	select A.*, B.*
	from tempfile2 as A
	left join
	temp.ip_readmission_exclusions_loaded as B
	on A.drg_num=B.drg_num;
quit;

data temp.ach_loaded&year. (keep = bene_id drg_cd ach_: excl_ach);
	set tempfile3 (rename = (admsn_dt = ach_admsndt dschrgdt = ach_dschrgdt 
	payment = ach_pay oop = ach_oop));
	ach_los = max(ach_dschrgdt - ach_admsndt, 1);	
	if missing(excl_ach) then excl_ach = 0;
run;

** LTCH IRF IPF**;

%macro merge_ip_cjr(filenames);
	%let filenamesn = %sysfunc(countw(&filenames.));
	%do i = 1 %to &filenamesn.;
	%let type = %scan(&filenames.,&i.);
	proc sql nowarn;
		create table tempfile as
		select A.*, B.*  
		from temp.ip_stay_level_loaded&year. (where=(&type. =1)) as A
		inner join
		temp.cjr_lejr_bene_list&year. as B
		on A.bene_id = B.bene_id;
	quit;

	data temp.&type._loaded&year. (keep = bene_id drg_cd &type._:)  ;
		set tempfile (rename = (admsn_dt = &type._admsndt
		dschrgdt = &type._dschrgdt payment = &type._pay oop = &type._oop));
		&type._los = max(&type._dschrgdt - &type._admsndt, 1);
	run;	

	%end;
%mend;

%merge_ip_cjr(ltch irf ipf);

** SNF **;
* First convert to stay level (similar to IP file) *;
* If discharge dates are missing, cap LOS at 100 (i.e. when Medicare coverage runs out) *;

proc sql nowarn;
	create table tempfile as
	select A.*, B.* 
	from mc.snf&year.clm as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id = B.bene_id;
quit;

proc sort data = tempfile out = tempfile2;
	by bene_id sadmsndt provider sdschrgdt;
quit;

data temp.snf_loaded&year. (keep = bene_id snf_:);
	set tempfile2 (rename = (sadmsndt = snf_admsndt
		sdschrgdt = snf_dschrgdt));
	by bene_id snf_admsndt provider snf_dschrgdt;
	if pmt_amt < 0 then pmt_amt = 0;
	oop_amt = ded_amt + coin_amt + blddedam ;
	
	retain snf_pay snf_oop;

	if first.provider then do;
		snf_pay = pmt_amt;
		snf_oop = oop_amt;
	end;
	else do;
		snf_pay = snf_pay + pmt_amt;
		snf_oop = snf_oop + oop_amt;
	end;
	
	if last.provider;
	
	snf_dschrg_impute_flag = missing(snf_dschrgdt);
	if missing(snf_dschrgdt) then snf_dschrgdt = snf_admsndt + 100;
	snf_los = snf_dschrgdt - snf_admsndt;
run;

** HHA HOS **;

%macro merge_hha_hos_cjr(filenames);
%let filenamesn = %sysfunc(countw(&filenames.));
%do i = 1 %to &filenamesn.;
%let type = %scan(&filenames.,&i.);
proc sql nowarn;
	create table tempfile as
	select A.*, B.*  
	from mc.&type.&year.clm as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id = B.bene_id;
quit;

data temp.&type._loaded&year. (keep = bene_id &type._:)  ;
	set tempfile (rename = (sfromdt = &type._admsndt sthrudt = &type._dschrgdt
	pmt_amt = &type._pay));
	if pmt_amt < 0 then pmt_amt = 0;
	&type._oop = 0;
	&type._los = &type._dschrgdt - &type._admsndt + 1;    
run;	

%end;
%mend;

%merge_hha_hos_cjr(hha hos);

** DME **;
* The out-of-pocket payment is in the line item file *;

proc sql nowarn;
	create table temp.dme_oop&year.
	as select clm_id, sum(ldedamt) as ldedamt, sum(coinamt) as coinamt
	from mc.dme&year.lnit (keep = clm_id ldedamt coinamt)
	group by clm_id;
quit;

proc sql nowarn;
	create table temp.tempfile as
	select A.*, B.*
	from mc.dme&year.clm as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id=B.bene_id;
quit;

proc sql nowarn;
	create table temp.tempfile2 as
	select A.*, B.*
	from temp.tempfile as A
	inner join
	temp.dme_oop&year. as B
	on A.clm_id=B.clm_id;
quit;

data temp.dme_loaded&year. (keep = bene_id dme_:) ;
    set temp.tempfile2 (rename = (sfromdt = dme_admsndt sthrudt = dme_dschrgdt pmt_amt = dme_pay));
	dme_oop = ldedamt + coinamt;
	if pmt_amt < 0 then pmt_amt = 0;
    dme_los = dme_dschrgdt - dme_admsndt + 1;
run;


** OP **;

* ER use in the OP file (used to generate ER visit flags in a later program) *;

data tempfile;
    set mc.op&year.revp1(keep = clm_id revpmt rev_cntr)
	mc.op&year.revp2(keep = clm_id revpmt rev_cntr)
	mc.op&year.revp3(keep = clm_id revpmt rev_cntr)
	mc.op&year.revp4(keep = clm_id revpmt rev_cntr)
	mc.op&year.revp5(keep = clm_id revpmt rev_cntr);
run; 	

data tempfile_er;
	set tempfile (keep = clm_id rev_cntr);
	ER_visit = 0;
	if rev_cntr in ("0450", "0451", "0452", "0453", "0454", "0455",
		"0456", "0457", "0458", "0459", "0981") then ER_visit = 1;
run;

proc sql nowarn;
	create table temp.opr_er&year. as
	select clm_id, max(ER_visit) as ER_visit
	from tempfile_er
	group by clm_id;
quit;

* Load OP utilization *;

data tempfile;
    set mc.op&year.clmp1(keep = clm_id bene_id sfromdt sthrudt pmt_amt ptb_ded ptb_coin blddedam)
	mc.op&year.clmp2(keep = clm_id bene_id sfromdt sthrudt pmt_amt ptb_ded ptb_coin blddedam)
	mc.op&year.clmp3(keep = clm_id bene_id sfromdt sthrudt pmt_amt ptb_ded ptb_coin blddedam)
	mc.op&year.clmp4(keep = clm_id bene_id sfromdt sthrudt pmt_amt ptb_ded ptb_coin blddedam)
	mc.op&year.clmp5(keep = clm_id bene_id sfromdt sthrudt pmt_amt ptb_ded ptb_coin blddedam);
run; 	

proc sql nowarn;
	create table tempfile2 as
	select A.*, B.*
	from tempfile as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id=B.bene_id;
quit;
	
data tempfile3;
	set tempfile2 (rename  = (sfromdt = op_admsndt
			sthrudt = op_dschrgdt
			pmt_amt = op_pay));
	if op_pay<0 then op_pay = 0;
	
	op_oop = ptb_ded + ptb_coin + blddedam;
	
	keep bene_id clm_id op_: ;
	op_los = op_dschrgdt - op_admsndt + 1;
run;

proc sql nowarn;
	create table temp.op_loaded&year. as
	select A.*, B.*
	from tempfile3 as A
	left join
	temp.opr_er&year. as B
	on A.clm_id = B.clm_id;
quit;

** PTB/CAR **;
* Out-of-pocket spending for CAR is in the line item file *;

data tempfile;
    set mc.ptb&year.lnitp1(keep = clm_id ldedamt coinamt)
	mc.ptb&year.lnitp2(keep = clm_id ldedamt coinamt)
	mc.ptb&year.lnitp3(keep = clm_id ldedamt coinamt)
	mc.ptb&year.lnitp4(keep = clm_id ldedamt coinamt)
	mc.ptb&year.lnitp5(keep = clm_id ldedamt coinamt);
run;

proc sql nowarn;
	create table temp.car_oop&year. as
	select clm_id, sum(ldedamt) as ldedamt, sum(coinamt) as coinamt
	from tempfile 
	group by clm_id;
quit;

data tempfile;
	set mc.ptb&year.clmp1(keep = clm_id bene_id sfromdt sthrudt pmt_amt dgns_cd1)
	mc.ptb&year.clmp2(keep = clm_id bene_id sfromdt sthrudt pmt_amt dgns_cd1)
	mc.ptb&year.clmp3(keep = clm_id bene_id sfromdt sthrudt pmt_amt dgns_cd1)
	mc.ptb&year.clmp4(keep = clm_id bene_id sfromdt sthrudt pmt_amt dgns_cd1)
	mc.ptb&year.clmp5(keep = clm_id bene_id sfromdt sthrudt pmt_amt dgns_cd1);
	prncpal_dgns_cd = compress(dgns_cd1,".");
run;

proc sql nowarn;
	create table tempfile2 as
	select A.*, B.*
	from tempfile as A
	inner join
	temp.cjr_lejr_bene_list&year. as B
	on A.bene_id=B.bene_id;
quit;
	
proc sql nowarn;
	create table tempfile3 as
	select A.*, B.*
	from tempfile2  as A
	inner join
	temp.car_oop&year. as B
	on A.clm_id=B.clm_id;
quit;

data temp.ptb_excluded_icd(rename=(dgns_cd = prncpal_dgns_cd));
	set temp.ptb_excluded_icd9;
	dgns_cd = put(prncpal_dgns_cd, 7.);
	drop prncpal_dgns_cd;
run;

proc sort data =  temp.ptb_excluded_icd
	out = temp.ptb_excluded_icd;
	by prncpal_dgns_cd;
quit;

proc sort data =  tempfile3
	out = tempfile4;
	by prncpal_dgns_cd;
quit;

proc sql nowarn;

	create table tempfile5 as
	select A.*, B.*, case when A.prncpal_dgns_cd = B.prncpal_dgns_cd then 1 end as excl_ptb
	from tempfile4 as A
	left join
	temp.ptb_excluded_icd as B
	on A.prncpal_dgns_cd=B.prncpal_dgns_cd;

quit;

data temp.car_loaded&year.;
	set tempfile5 (rename = (sfromdt = car_admsndt
        sthrudt = car_dschrgdt
        pmt_amt = car_pay));
	if pmt_amt<0 then pmt_amt = 0;
	car_oop = ldedamt + coinamt;
	car_los = car_dschrgdt - car_admsndt + 1;
	if missing(excl_ptb) then excl_ptb = 0;
	keep bene_id prncpal_dgns_cd car_: excl_ptb;
run;

*********************************************************;
*              ALL CAUSE READMISSIONS                   *;
*********************************************************;
*                                                       *;
* Generate flag for all cause readmission within episode*;
*                                                       *;
*********************************************************;

proc sort data =  temp.ip_stay_level_loaded&year. (where = ( (prvdrnum <= 999 & ~missing(prvdrnum)) or (prvdrnum >= 1300 and prvdrnum <= 1399)))
	out = tempfile;
	by bene_id descending admsn_dt;
quit;

data temp.all_cause_readmission&year. (keep = bene_id provider admsn_dt all_cause_readmit);
	set tempfile;
	by  bene_id descending admsn_dt;
	retain next_admsn_dt;
	
	if first.bene_id then next_admsn_dt = .;
	all_cause_readmit = 0;
		
	if ~missing(next_admsn_dt) and next_admsn_dt <= dschrgdt + 89 then all_cause_readmit = 1;
		
	next_admsn_dt = admsn_dt;
run;


***********;
*ER VISITS*;
***********;

data tempfile;
	set mc.ip&year.rev;
	ER_visit = 0;
	if rev_cntr in ("0450", "0451", "0452", "0453", "0454", "0455",
					"0456", "0457", "0458", "0459", "0981") then ER_visit = 1;
	if ER_visit = 1;
	keep clm_id ER_visit; 
run;

proc sort nodupkey data = tempfile out = temp.ipr_er&year.;
	by ER_visit clm_id;
quit;

data tempfile;
	set mc.ip&year.clm;
	prvdstr = substr(provider, 3, 4);
	if anyalpha(prvdstr) = 0 then prvdrnum = input(prvdstr, 4.); 

	ach  = ((prvdrnum <= 999 & ~missing(prvdrnum)) | (prvdrnum >= 1300 & prvdrnum <= 1399))  ;
	
	if drg_cd in ("469", "470") and ach = 1 then delete;
run;

proc sql nowarn;
	create table tempfile2 as 
	select A.*, B.ER_visit 
	from tempfile as A
	inner join
	temp.ipr_er&year. as B
	on A.clm_id = B.clm_id;
quit;

data temp.ip_er_temp&year.;
	set tempfile2(keep=bene_id ER_visit sadmsndt sdschrgdt
	rename = (sadmsndt = ER_admsndt sdschrgdt = ER_dschrgdt));
run;

data temp.op_er_temp&year.;
	set temp.op_loaded&year. (keep=bene_id ER_visit op_admsndt op_dschrgdt
	rename = (op_admsndt = ER_admsndt op_dschrgdt = ER_dschrgdt));
	if ER_visit = 1;
run;

data tempfile;
	set temp.ip_er_temp&year. temp.op_er_temp&year.;
run;
proc sort nodupkey data = tempfile out = temp.ER_loaded&year.;
	by bene_id ER_admsndt descending ER_dschrgdt;
quit;

*******************************************************************;
*                     DEFINE CJR LEJR EPISODES                    *;
*******************************************************************;
*                                                                 *;
* Create an episode level data set by merging together the index  *;
* admissions with other utilization in episode                    *;
*                                                                 *;
* (1) For each type of utilization (ACH LTCH IRF IPF SNF DME HOS  *;
* OP CAR ER), do the following:                                   *;
* (a) Keep if utilization falls within episode                    *;
* (b) Prorate payment and LOS if utilization extends beyond the   *;
* episode. This differs by type of utilization and is coded based *;
* on the federal register.                                        *;
* (c) Collapse to episode level                                   *;
*                                                                 *;
* (2) Merge all utilization within episode with index admission   *;
*                                                                 *;
* (3) Create dataset of (CJR-eligible and ineligible) LEJR        *; 
* episodes. Create key spending and discharge variables.          *;
*                                                                 *;
* This final dataset contains flags for MSA eligibility (OrigElig)*;
* and within MSA eligibility (drop_XX).                           *;
*                                                                 *;
*******************************************************************;

** KEEP UTILIZATION THAT FALLS WITHIN EPISODE **;

%macro typelist_prog1(filenames);
	%let filenamesn = %sysfunc(countw(&filenames.));
	%do i = 1 %to &filenamesn.;
	%let type = %scan(&filenames.,&i.);
	proc sql nowarn;
		create table tempfile as
		select A.*, B.*
		from temp.&type._loaded&year. as A
		inner join
		temp.cjr_lejr_index_admissions&year. as B
		on A.bene_id = B.bene_id;
	quit;

	data temp.&type._in_episode_id_&year.;
		set tempfile;
		if &type._admsndt >= admsn_dt and &type._admsndt <= epsdenddt;
	run;
	%end;
%mend;

%typelist_prog1(ach ltch irf ipf snf dme hos op car ER);

%let type = hha;

proc sql nowarn;
	create table tempfile as
	select A.*, B.*
	from temp.&type._loaded&year. as A
	inner join
	temp.cjr_lejr_index_admissions&year. as B
	on A.bene_id = B.bene_id;
quit;

* HHA can start before the episode and overlap with the episode *;

data temp.&type._in_episode_id_&year.;
	set tempfile;
	if (&type._admsndt >= admsn_dt and &type._admsndt <= epsdenddt)
		or (&type._admsndt < admsn_dt & &type._dschrgdt >= admsn_dt) ;
run;

proc sql nowarn;
	create table temp.ER_in_episode_&year. as
	select episode_id, max(ER_visit) as ER_visit
	from temp.er_in_episode_id_&year.
	group by episode_id;
quit;


** PRORATE PAYMENT AND LOS IF UTILIZATION EXTENDS BEYOND EPISODE **;

* LTCH IRF IPF SNF DME HOS: prorate by the share that falls within episode *;

%macro typelist_prog2(filenames);
	%let filenamesn = %sysfunc(countw(&filenames.));
	%do i = 1 %to &filenamesn.;
	%let type = %scan(&filenames.,&i.);
	data tempfile;
		set temp.&type._in_episode_id_&year.;
		prorate_flag = (epsdenddt < &type._dschrgdt);
		if prorate_flag = 1 then do; 
			prorate_los = &type._los - (&type._dschrgdt - epsdenddt);
			prorate_pay = (prorate_los / &type._los) * &type._pay;
			prorate_oop = (prorate_los / &type._los) * &type._oop;
			&type._los = prorate_los ;
			&type._pay = prorate_pay ;
			&type._oop = prorate_oop ;
		end;
	run;

	proc sql;
		create table temp.&type._in_episode_&year. as
		select episode_id, sum(&type._pay) as &type._pay, sum(&type._los) as &type._los,
		 sum(&type._oop) as &type._oop 
		from tempfile
		group by episode_id;
	quit;
	%end;
%mend;

%typelist_prog2(ltch irf ipf snf dme hos);

* HHA: (unlike other services) can begin before episode begins *;

%let type = hha;

data tempfile;
	set temp.&type._in_episode_id_&year.;
	
	prorate_flag_dschrg = (epsdenddt < hha_dschrgdt);
	prorate_flag_admsn = (hha_admsndt < admsn_dt);
	if prorate_flag_dschrg = 1 & prorate_flag_admsn = 0 then 
		prorate_los = hha_los - (hha_dschrgdt - epsdenddt);
		
	if prorate_flag_dschrg = 0 & prorate_flag_admsn = 1 then   
        prorate_los = hha_los - (admsn_dt - hha_admsndt);
    
	if prorate_flag_dschrg = 1 & prorate_flag_admsn = 1 then
		prorate_los = hha_los - (hha_dschrgdt - epsdenddt) - (admsn_dt - hha_admsndt);
    
	if (prorate_flag_dschrg = 1 | prorate_flag_admsn = 1) then do;
		prorate_pay = (prorate_los / hha_los) * hha_pay;
		hha_los = prorate_los;
		hha_pay = prorate_pay;
	end;	 
run;

proc sql nowarn;
	create table temp.hha_in_episode_&year. as
	select episode_id, sum(&type._pay) as &type._pay, sum(&type._los) as &type._los,
	 sum(&type._oop) as &type._oop 
	from tempfile
	group by episode_id;
quit;

* ACH: prorate by geometric LOS (gmlos) and count the first day twice *;

%let type = ach;

data temp.&type._in_episode_id_&year._temp;
	set temp.&type._in_episode_id_&year.(rename=(drg_cd=drg_cd_str));
	drg_cd = input(drg_cd_str, 8.);
run;

proc sql nowarn;
	create table tempfile as
	select A.*, B.* 
	from temp.&type._in_episode_id_&year._temp as A
	left join  temp.drg_gmlos2015 as B
	on A.drg_cd = B.drg_cd;
quit;

data tempfile2;
	set tempfile;
	prorate_flag = (epsdenddt < ach_dschrgdt);
	if prorate_flag = 1 then do;	
		prorate_los = ach_los - (ach_dschrgdt - epsdenddt);
		ach_los = prorate_los ;
	end;
	if prorate_flag = 1 & prorate_los < gmlos & ~missing(gmlos) then do;
		prorate_pay = ach_pay/gmlos * (1+prorate_los);
		prorate_oop = ach_oop/gmlos * (1+prorate_los);
        ach_pay = prorate_pay ;
        ach_oop = prorate_oop ;
	end;
run;

proc sql nowarn;
	create table temp.&type._in_episode_&year. as
	select episode_id, excl_ach, sum(&type._pay) as &type._pay, sum(&type._los) as &type._los,
	 sum(&type._oop) as &type._oop
	from tempfile2 (where = (excl_ach=0))
	group by episode_id, excl_ach;
quit;

proc sql nowarn;
	create table temp.excl_ach_in_episode_&year. as
	select episode_id, excl_ach, sum(&type._pay) as excl_&type._pay, sum(&type._los) as excl_&type._los,
	 sum(&type._oop) as excl_&type._oop
	from tempfile2 (where = (excl_ach=1))
	group by episode_id, excl_ach;
quit;


* OP: prorate by LOS *;
* [Segment commented out was not run] variable not available in views when the code was written *;
	
%let type = op;

data tempfile;
	set temp.&type._in_episode_id_&year.;
	
	prorate_flag = epsdenddt < op_dschrgdt;
	if prorate_flag = 1 then do;
		prorate_los = op_los - (op_dschrgdt - epsdenddt) ;
        prorate_pay = (prorate_los / op_los) * op_pay; 
        prorate_oop = (prorate_los / op_los) * op_oop; 
	*prorate_exclpay = (prorate_los / op_los) * exclpay_medv_passthru;
        op_los = prorate_los;
        op_pay = prorate_pay;
        op_oop = prorate_oop;
	*exclpay_medv_passthru = prorate_exclpay;
    end;
run;

proc sql nowarn;
	create table temp.&type._in_episode_&year. as
	select episode_id, sum(&type._pay) as &type._pay, sum(&type._los) as &type._los,
	sum(&type._oop) as &type._oop
	from tempfile
	group by episode_id;
quit;

* PTB/CAR: prorate by LOS *;

%let type = car;

data tempfile;
	set temp.&type._in_episode_id_&year.;
	
	prorate_flag = epsdenddt < car_dschrgdt;

	if prorate_flag = 1 then do;
		prorate_los = car_los - (car_dschrgdt - epsdenddt) ;
        	prorate_pay = (prorate_los / car_los) * car_pay; 
        	prorate_oop = (prorate_los / car_los) * car_oop; 
        	car_los = prorate_los;
        	car_pay = prorate_pay;
        	car_oop = prorate_oop;
	end;
run;

* Included CAR services *;
proc sql;
	create table temp.&type._in_episode_&year. as
	select episode_id, sum(&type._pay) as &type._pay, sum(&type._los) as &type._los,
	 sum(&type._oop) as &type._oop
	from tempfile (where = (excl_ptb = 0))
	group by episode_id;
quit;

* Excluded CAR services *;
proc sql;
	create table temp.excl_&type._in_episode_&year. as
	select episode_id, sum(&type._pay) as excl_&type._pay, sum(&type._los) as excl_&type._los,
	 sum(&type._oop) as excl_&type._oop
	from tempfile (where = (excl_ptb = 1))
	group by episode_id;
quit;


** MERGE CJR INDEX ADMISSIONS WITH OTHER FILES **;

%let type = ach;
proc sql nowarn;
	create table tempfile as
	select A.*, B.*
	from temp.cjr_lejr_index_admissions&year. as A
	left join temp.&type._in_episode_&year. as B
	on A.episode_id = B.episode_id;
run;

%macro typelist_prog3(filenames);
	%let filenamesn = %sysfunc(countw(&filenames.));
	%do i = 1 %to &filenamesn.;
	%let type = %scan(&filenames.,&i.);
	proc sql nowarn undo_policy = none;
		create table tempfile as
		select A.*, B.*
		from tempfile as A
		left join temp.&type._in_episode_&year. as B
		on A.episode_id = B.episode_id;
	run;
	%end;
%mend;

%typelist_prog3(ltch irf ipf snf hha dme hos op car excl_ach excl_car);


proc sql nowarn;
	create table tempfile2 as
	select A.*, B.*
	from tempfile as A
	left join temp.ER_in_episode_&year. as B
	on A.episode_id = B.episode_id;
run;

proc sql nowarn;
	create table tempfile3 as
	select A.*, B.*
	from tempfile2 as A
	left join temp.all_cause_readmission&year. as B
	on A.bene_id = B.bene_id and A.provider = B.provider and A.admsn_dt = B.admsn_dt ;
run;

data output.all_episode_level_data&year. ;
	set tempfile3;

	* FILL IN THE ZEROS FOR SPENDING, LOS, AND ER *;

	array payarr {*} ach_pay ltch_pay irf_pay ipf_pay snf_pay hha_pay dme_pay hos_pay op_pay car_pay excl_ach_pay excl_car_pay;
	array losarr {*} ach_los ltch_los irf_los ipf_los snf_los hha_los dme_los hos_los op_los car_los excl_ach_los excl_car_los;
	array ooparr {*} ach_oop ltch_oop irf_oop ipf_oop snf_oop hha_oop dme_oop hos_oop op_oop car_oop excl_ach_oop excl_car_oop;
	
	do i = 1 to 12 ;
		if missing(payarr[i]) then payarr[i] = 0;
		if missing(losarr[i]) then losarr[i] = 0;
		if missing(ooparr[i]) then ooparr[i] = 0;
	end;
	
	if ER_visit = . then ER_visit = 0 ;

	* [The following line was not run] variable not available when code was written *;
	*if exclpay_medv_passthru = . then  exclpay_medv_passthru = 0 ;

	* TOTAL MEDICARE PAYMENT *;

	tot_pay = index_pay + ach_pay + ltch_pay + irf_pay + ipf_pay 
          + car_pay + snf_pay + hha_pay + dme_pay + hos_pay + op_pay;

	* TOTAL OUT-OF-POCKET PAYMENT *;

	tot_oop = index_oop + ach_oop + ltch_oop + irf_oop + ipf_oop
          + car_oop + snf_oop + hha_oop + dme_oop + hos_oop + op_oop;

	* TOTAL MEDICARE PAYMENT (EXCL. INDEX ADMISSION) *;

	tot_nonindex_pay =  ach_pay + ltch_pay + irf_pay + ipf_pay
          + car_pay + snf_pay + hha_pay + dme_pay + hos_pay + op_pay;
			
	* POST ACUTE CARE (PAC) PAYMENT	*;

	pac_pay = ltch_pay + irf_pay + snf_pay + hha_pay;
		
	* INSTITUTIONAL POST ACUTE CARE PAYMENT *;

	inst_pac_pay = ltch_pay + irf_pay + snf_pay;
	
	* LENGTH OF STAY AT INSTITUTIONAL POST ACUTE CARE *;

	inst_pac_los = ltch_los + irf_los + snf_los;

	* INDICATOR FOR CJR SAMPLE *;

	cjr_sample = (drop_pps = 0 & drop_bpci = 0 & drop_partAB = 0 & drop_esrd = 0 & 
        drop_hmo = 0 & drop_mcareprimary = 0 & drop_readmit = 0 & drop_die = 0);

	* INDICATOR FOR MEDICAID STATUS *;

	mcaid = (buyin_mo ~= 0);
	drop episode_mo: buyin1-buyin12 hmo1-hmo12 ;
	
	* INDICATORS FOR DISCHARGES TO PAC AND INST PAC	*;

	dschrg_pac = (stus_cd in ("03", "06", "62", "63", "83", "86", "90", "91"));
	dschrg_inst_pac = (stus_cd in ("03", "62", "63", "83", "90", "91")) ;
	if dschrg_pac = 1 then inst_pac_if_any_pac = dschrg_inst_pac ;

	drop ach excl_ach ipf irf ltch i;
	
run;

%mend;