version 15
set more off
preliminaries, seed(02138)
global ext = "[EXT DIR]"
global data_dir = "[DATA DIR]"

global Outcomes = "tot_spend inst_pac_spend dschrg_inst_pac"
global pat_chars = "age female white major_comps disabled elixsum hip_fracture"

program main

    load_data
	slopes_table
	bivariate_regs
    
end


program load_data

    use "$ext/Analysis Data/output/episode_level_data2016_2017.dta", clear
    merge m:1 provider using  "$ext/Analysis Data/output/provider_level_data2016_2017.dta", ///
        keep(3) keepusing(mandatory num_epsd_pre) nogen
		
	* Keep only providers from the baseline
    merge m:1 provider using "$ext/Analysis Data/output/provider_level_data_baseline", ///
        keep(3) keepusing(provider) nogen
		
	* Merge with Elixhauser
    merge m:1 episode_id using "$ext/Elixhauser/output/elix2016_2017.dta", ///
        assert(2 3) keep(3) keepusing(elixsum) nogen
		
    qui summ bene_age, det
	local mincut = `r(min)' - 1
	local maxcut = `r(max)' + 1
    egen age_bins = cut(bene_age), at(`mincut' 55 60 65 70 75 80 85 90 `maxcut')
    destring bene_race, replace
    gen female = (bene_sex == "2")
	gen white = (bene_race == 1)
    gen major_comps = (drg_cd == "469")
    gen disabled = (bene_orec == "1"|bene_orec == "3")

	collapse (mean) age female white major_comps disabled /// 
        elixsum hip_fracture, by(provider)
	save "../temp/patient_chars", replace
	
	use "$ext/Analysis Data/output/provider_level_data2016_2017.dta", clear
		
	* Keep only providers from the baseline
    merge 1:1 provider using "$ext/Analysis Data/output/provider_level_data_baseline", ///
        keep(3) keepusing(provider) nogen
		
    gen voluntary = mandatory == 0
    
    merge 1:1 provider using "$ext/AHA/output/aha", assert(1 2 3) ///
        keep(1 3) nogen
	gen ownership = 1 if prov_fp == 1
	replace ownership = 2 if prov_np == 1
	replace ownership = 3 if ownership == .
			
	* Merge in Pat Chars
    merge 1:1 provider using "../temp/patient_chars", ///
        assert(2 3) keep(3) nogen	
		
    * Merge with CQS
    destring provider, replace
    merge 1:1 provider using "$ext/CQS/output/quality_measures.dta", ///
        assert(1 2 3) keep(1 3) keepusing(cqs) nogen

		
	foreach var in $pat_chars num_epsd_pre cqs prov_beds ownership prov_coth strata msa {
    	gen bp_`var' = `var' * Treatment
    }
    bys Treatment: gen Num = _n
    replace Num = 0 if Treatment == 0
    qui sum Num if Treatment == 1, d
    local num_treat_hosp = r(max)
    forv i = 1/`num_treat_hosp' {
    	gen Treat_fe`i' = (Num == `i' & Treatment == 1)
    }
    save "../temp/slopes_data_loaded", replace

end

program slopes_table
    * Table A5
    local hosp_chars num_epsd_pre cqs prov_beds prov_coth i.ownership i.strata i.msa

    local counter = 0
    foreach outcome in $Outcomes {
      use ../temp/slopes_data_loaded, clear
      local counter = `counter' + 1
      
      slope_sd, y(`outcome') fe(0) x("") ///
        matname(hosp_slope_sd_`counter') counter(`counter')
	
    foreach var in `hosp_chars ' {
	if "`var'" == "i.msa" {
            slope_sd, y(`outcome') matname(hosp_slope_sd_`counter') ///
	        absorb(`var') fe(1) counter(`counter')
        }
	if "`var'" != "i.msa" {
        slope_sd, y(`outcome') matname(hosp_slope_sd_`counter') ///
        x(`var') fe(0) counter(`counter')
	}
    }
 
    slope_sd, y(`outcome') matname(hosp_slope_sd_`counter') ///
        x("num_epsd_pre cqs prov_beds i.ownership prov_coth") absorb(bp_msa) fe(1) ///
	    counter(`counter')
    slope_sd, y(`outcome') matname(hosp_slope_sd_`counter') ///
        x("num_epsd_pre cqs prov_beds i.ownership prov_coth $pat_chars") ///
	    absorb(bp_msa) fe(1) counter(`counter')
    }
	
	 matrix hosp_slope_sd = hosp_slope_sd_1 \ hosp_slope_sd_2 \ hosp_slope_sd_3 \ hosp_slope_sd_1_n
  
 
      matrix_to_txt, saving(../output/tables/hosp_slope_sd.txt) mat(hosp_slope_sd) ///
        format(%20.4f) title(<tab:hosp_slope_sd>) replace
      matrix list hosp_slope_sd
	  clear
	  svmat hosp_slope_sd
    
end


program slope_sd 
  
  syntax, y(str) matname(str) fe(int) counter(int) [absorb(str) x(str)] 
  qui sum Num if Treatment == 1, d
  local num_treat_hosp = r(max)
  
  if `fe' == 1 qui reghdfe `y'_post `x' `y'_2013 `y'_2014 i.strata ///
    [aw = num_epsd_post], absorb(`absorb', savefe)
  if `fe' == 0 qui reg `y'_post `x' `y'_2013 `y'_2014 i.strata ///
    [aw = num_epsd_post], robust
  predict resid, residuals
  qui reg resid `y'_2013 `y'_2014 i.strata Treat_fe*, robust
  qui gen hosp_slope = _b[Treat_fe1] if Treat_fe1 == 1
  forv i = 2/`num_treat_hosp' {
      qui replace hosp_slope = _b[Treat_fe`i'] if Treat_fe`i' == 1
  }
  qui sum hosp_slope if hosp_slope != . [aw = num_epsd_post], d
  local stdev = r(sd)
  local num_hosp = r(N)
  matrix `matname' = nullmat(`matname') , `stdev'
  if `counter' == 1 matrix `matname'_n = nullmat(`matname'_n) , `num_hosp'
  use ../temp/slopes_data_loaded, clear
end

program bivariate_regs
    * Table 2
    use "../temp/slopes_data_loaded", clear
    qui sum Num if Treatment == 1, d
    local num_treat_hosp = r(max)
	foreach outcome in $Outcomes {
    	reg `outcome'_post `outcome'_2013 `outcome'_2014 i.strata Treat_fe*, robust
	
	    qui gen hosp_slope_`outcome' = _b[Treat_fe1] if Treat_fe1 == 1
        forv i = 2/`num_treat_hosp' {
            qui replace hosp_slope_`outcome' = _b[Treat_fe`i'] if Treat_fe`i' == 1
        }
		summ hosp_slope_`outcome' [aw = num_epsd_post]
		matrix biv_slope_`outcome'  = nullmat(biv_slope_`outcome') \ (`r(mean)', `r(sd)')

		foreach xvar in num_epsd_pre cqs prov_beds prov_coth {
	        reg hosp_slope_`outcome' `xvar' [aw = num_epsd_post], robust
		    matrix biv_slope_`outcome'  = nullmat(biv_slope_`outcome') \ ( _b[`xvar'], _se[`xvar'])
		}
		reg hosp_slope_`outcome' prov_fp prov_np [aw=num_epsd_post], robust
			matrix biv_slope_`outcome' = nullmat(biv_slope_`outcome') \ (_b[prov_fp], _se[prov_fp])
			matrix biv_slope_`outcome' = nullmat(biv_slope_`outcome') \ (_b[prov_np], _se[prov_np])
	}
	matrix biv_slopes = biv_slope_tot_spend, biv_slope_inst_pac_spend, biv_slope_dschrg_inst_pac
    mat list biv_slopes
end

* Execute
main
